Package: StructuralVariantAnnotation
Type: Package
Title: Variant annotations for structural variants
Version: 1.29.0
Date: 2024-04-23
Authors@R: c(
	person("Daniel", "Cameron", email="daniel.l.cameron@gmail.com", role=c("aut", "cre"), comment=c(ORCID = "0000-0002-0951-7116")),
	person("Ruining", "Dong", role=c("aut"), comment=c(ORCID = "0000-0003-1433-0484")))
Description: StructuralVariantAnnotation provides a framework for
	analysis of structural variants within the Bioconductor ecosystem.
	This package contains 
	contains useful helper
	functions for dealing with structural variants in VCF format.
	The packages contains functions for parsing VCFs from a number
	of popular callers as well as functions for dealing with 
	breakpoints involving two separate genomic loci encoded as
	GRanges objects.
License: GPL-3 + file LICENSE
Depends: GenomicRanges, rtracklayer, VariantAnnotation, BiocGenerics, R
        (>= 4.1.0)
Imports: assertthat, Biostrings, pwalign, stringr, dplyr, methods,
        rlang, GenomicFeatures, IRanges, S4Vectors,
        SummarizedExperiment, GenomeInfoDb,
Suggests: ggplot2, devtools, testthat (>= 2.1.0), roxygen2, rmarkdown,
        tidyverse, knitr, ggbio, biovizBase,
        TxDb.Hsapiens.UCSC.hg19.knownGene, BSgenome.Hsapiens.UCSC.hg19,
RoxygenNote: 7.1.1
Encoding: UTF-8
VignetteBuilder: knitr
biocViews: DataImport, Sequencing, Annotation, Genetics,
        VariantAnnotation
git_url:
        https://git.bioconductor.org/packages/StructuralVariantAnnotation
git_branch: devel
git_last_commit: aaa2e09
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 23:36:30 UTC; biocbuild
Author: Daniel Cameron [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0951-7116>),
  Ruining Dong [aut] (ORCID: <https://orcid.org/0000-0003-1433-0484>)
Maintainer: Daniel Cameron <daniel.l.cameron@gmail.com>
Built: R 4.6.0; ; 2026-05-19 05:11:27 UTC; unix
