# Generated by roxygen2: do not edit by hand

export(breakendRanges)
export(breakpointRanges)
export(breakpointgr2bedpe)
export(breakpointgr2pairs)
export(calculateReferenceHomology)
export(countBreakpointOverlaps)
export(extractBreakpointSequence)
export(extractReferenceSequence)
export(findBreakpointOverlaps)
export(findInsDupOverlaps)
export(findTransitiveCalls)
export(hasPartner)
export(isStructural)
export(isSymbolic)
export(numtDetect)
export(pairs2breakpointgr)
export(partner)
export(rtDetect)
export(simpleEventLength)
export(simpleEventType)
import(BiocGenerics)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(SummarizedExperiment)
import(VariantAnnotation)
import(rtracklayer)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,nchar)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,subseq)
importFrom(dplyr,"%>%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(pwalign,deletion)
importFrom(pwalign,insertion)
importFrom(pwalign,nindel)
importFrom(pwalign,nucleotideSubstitutionMatrix)
importFrom(pwalign,pairwiseAlignment)
importFrom(rlang,.data)
